using System;
using System.Collections.Generic;
using System.Linq;
using Magify.Model;
using JetBrains.Annotations;

namespace Magify
{
    internal class ContentProvider : IContextListener
    {
        private const string TagEarliest = "earliest";
        private const string TagLatest = "latest";

        [CanBeNull]
        private Dictionary<string, ContentGroup> _content;

        public bool HasContent => _content != null;

        public void UpdateContext(CampaignsContext context, ContextKind kind)
        {
            switch (kind)
            {
                case ContextKind.Default when !HasContent:
                case ContextKind.Saved or ContextKind.Downloaded:
                    _content = context.Content;
                    break;
            }
        }

        public void ResetContentMap()
        {
            _content = default;
        }

        [NotNull]
        public List<ContentItem> GetContentList(string groupKey, string contentKey, ICollection<string> tags = null)
        {
            if (_content == null || !_content.TryGetValue(groupKey, out var group) || group == null || group.Items == null)
            {
                return new List<ContentItem>();
            }
            return group.Items.Where(itemFilter).Select(ModelToContentItem).ToList();

            bool itemFilter(Model.ContentItem it)
            {
                if (it.Key != contentKey || it.Tags == null)
                {
                    return false;
                }

                return tags == null || tags.All(it.Tags.Contains);
            }
        }

        [CanBeNull]
        public ContentItem GetEarliestContent(string groupKey, string contentKey)
        {
            return GetContentItem(groupKey, contentKey, TagEarliest);
        }

        [CanBeNull]
        public ContentItem GetLatestContent(string groupKey, string contentKey)
        {
            return GetContentItem(groupKey, contentKey, TagLatest);
        }

        [CanBeNull]
        private ContentItem GetContentItem(string groupKey, string contentKey, string tag)
        {
            if (_content == null || !_content.TryGetValue(groupKey, out var group) || group == null)
            {
                return null;
            }
            var contentList = GetContentList(groupKey, contentKey);
            return contentList.Count switch
            {
                1 => contentList.First(),
                _ => contentList.FirstOrDefault(c => c.Tags.Contains(tag))
            };
        }

        private ContentItem ModelToContentItem(Model.ContentItem model) => new ContentItem
        {
            Key = model.Key,
            Link = model.Key,
            Previews = model.Previews,
            Tags = model.Tags
        };
    }
}